
#if WORDSIZE == 32
#define lsode dlsode
#define b22in db22in
#define bs2dr dbs2dr
#define bs2vl dbs2vl
#define bs2in dbs2in
#define bs3in dbs3in
#define bs3vl dbs3vl
#define bsnak dbsnak
#define isamax idamax
#define sasum dasum
#define saxpy daxpy
#define scopy dcopy
#define sdot ddot
#define sgbco dgbco
#define sgbfa dgbfa
#define sgbsl dgbsl
#define sgefa dgefa
#define sgesl dgesl
#define snrm2 dnrm2
#define sscal dscal
#define ssum dsum
#define sswap dswap
#define ISAMAX idamax
#define SASUM dasum
#define SAXPY daxpy
#define SCOPY dcopy
#define SDOT ddot
#define SSCAL dscal
#define SSWAP dswap
#define SNRM2 dnrm2
#define SGBFA dgbfa
#define SGBSL dgbsl
#define SGEFA dgefa
#define SGESL dgesl
#define SNRM2 dnrm2
#define EWSET dewset
#define XERRWV dxerrwv
#define dfloat real
#define float real
#endif
